from django.shortcuts import get_object_or_404, render
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.views import generic

############# SOCKET COMMUNICATION ##############################
import socket

def socket_receive(sock):
    total_data=[]
    data=''
    EndCommSymbol = "\b"
    
    while True:
            data=sock.recv(8192)
            if EndCommSymbol in data:
                total_data.append(data[:data.find(EndCommSymbol)])
                break
            total_data.append(data)
            if len(total_data)>1:
                last_pair=total_data[-2]+total_data[-1]
                if EndCommSymbol in last_pair:
                    total_data[-2]=last_pair[:last_pair.find(EndCommSymbol)]
                    total_data.pop()
                    break

    return ''.join(total_data)


def send_message(message):
    port = 9901
    try:
        # connect with the server
        sock = socket.socket()
        sock.connect(("localhost", port))

        #send the entire message with the \b end symbol at the end (the expected final symbol in the server)
        sock.sendall(" " + message + "\b")

        # read the response
        res = socket_receive(sock).strip().replace("<", "'").replace(">", "'").replace("\n", "<br>")
        # res = '<table class="tweb"><tr><td>' + res +  '</td></tr></table>'

        # close the communication    
        sock.close()
    except:
        res =  "Error: the ASERYLA service is not listening (expected port " + str(port) + ")"

    return res





############ HTML PETITIONS #############################



def ask(request):
    question = request.GET.get("question", "")

    # transform the options into the expected parameters
    ds = request.GET.get("ds", "")
    if ds == "on":
       ds = 'checked="checked"'
    vds = "1"
    if len(ds) == 0:
        vds = "0"
    
    gm = request.GET.get("gm", "")
    if gm == "on":
        gm = 'checked="checked"'
    vgm = "1"
    if len(gm) == 0:
        vgm = "0"
        
    gt = request.GET.get("gt", "0")
    
    gx = request.GET.get("gx", "0")
    
    gp = request.GET.get("gp", '')
    if gp == "on":
        gp = 'checked="checked"'
    vgp = "1"
    if len(gp) == 0:
        vgp = "0"

    params = vds + " " + vgm + " " + gt + " " + gx + " " + vgp + " 1 1 0 0 0 " 

    
    res = ""
    if len(question)>0:
        # send the order with the asked question and the indicated parameters
       res = send_message("?q " + params + question)
       
       # affirmative question are in an special format, and is asked for confirmation
       if res[0] == '?':
           order = res[3:] + " 1 3 2 "
           r = res[1]
           res = "No"
           if r == 'u': res = "Unknown"
           if r == 'y': res = "Yes"

           res = res + ' &nbsp;&nbsp; right? <a href="conf/' + order + '1/">Yes</a> <a href="conf/' + order + '0/">No</a> '
       else:
            # the elements of the results (if are concepts) are presented with a link to explore them
           
            parts = res.split(',')
            res = ""
            for p in parts:
                pp = p.strip()
                # not transform nothing that is not a concept
                if not p.isdigit() and p != "None" and p[0:13] != "Misunderstand" and p[0:5] != "Error":
                    pn = pp
                    pr = ""
                    if pp[-1] == '%': # the hitting percentage can't be included in the link
                        perc = pp.rfind(' ')
                        pn = pp[0:perc]
                        pr = pp[perc:]
                    
                    pp = '<a href="/web/explore?word=' + pn + '"">' + pn + "</a>" + pr
                    
                res = res + pp + ", "
                
            res = res[0:len(res)-2]   
                
    return render(request, 'ask.html', {'inp': question, 'res': res, 'ds': ds, 'gm': gm, 'gt': gt, 'gx': gx, 'gp': gp } )


# send a question confirmation order and return to the "ask" page
def conf(request, ilc):
    res = send_message("?c " + ilc)
    return ask(request)



stats = ""

def explore(request):
    word = request.GET.get("word", "")
    
    res = ""
    if len(word) > 0:
        ofs = word.split('of')
        word = ""
        for o in ofs:  # convert the ofs into the expected % format
            word = o.strip() + "%" + word  

        # adapt the specializations
        word = word.strip()[0:len(word)-1].replace(" ", "_")
        
        res = send_message("@s " + word)
    else:
        # the first time show the stats (number of words, relations and sentences availables in the memory)
        global stats
        stats = send_message("@a")
        if stats[0:6] == "Error:":
            stats = ""
   
    return render(request, 'explore.html', {'inp': word, 'res': res, 'stats': stats} )




def teach(request):
    snt = request.GET.get("sentence", "")
    
    res = ""
    if len(snt)>0:
        # simply sends the order and shows directly the response
       res = send_message("Tl 1 3 " + snt.strip())
    
    return render(request, 'teach.html', {'inp': snt, 'res': res} )






